/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * LaserCalc
 * Copyright (C) Udo Eisenbarth 2008 <udo.eisenbarth@web.de>
 * 
 * LaserCalc is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * LaserCalc is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "gui-add-element.hpp"
#include <iostream>
#include <map>
#include <iterator>
#include <string>
#include "lens.hpp"
#include "distance.hpp"
#include "util.hpp"

using namespace std;

GuiAddElement::GuiAddElement(BaseObjectType* cobject,
                const Glib::RefPtr<Gnome::Glade::Xml>& glade_xml)
:Gtk::VBox(cobject),
path(new OpticPath)
{
	#define GLADE_FILE "gui-addelement.glade"
	refXml=load_glade_xml (GLADE_FILE);
	
	refXml->reparent_widget("vbox1",*this);
	//connect the buttons
	refXml->connect_clicked("GuiAddElement::add",
		sigc::mem_fun(*this, &GuiAddElement::on_add_clicked));
	refXml->connect_clicked("GuiAddElement::remove",
		sigc::mem_fun(*this, &GuiAddElement::on_remove_clicked));
	// Element tree
	refXml->get_widget("GuiAddElement::elementlist",ElementTreeView);
	// Properties tree
	refXml->get_widget("GuiAddElement::proplist",propTreeView);
	
	proptree=new PropTree(propTreeView);
	proptree->signalPropertiesChanged().connect(
		sigc::mem_fun(*this, &GuiAddElement::on_prop_changed));									
	//Create the Tree model:
  	refElementTreeModel = Gtk::ListStore::create(columns);
  	ElementTreeView->set_model(refElementTreeModel);
	//Add the TreeView's view columns:
	ElementTreeView->append_column(_("#"), columns.id);
	ElementTreeView->append_column(_("type"), columns.type);
	ElementTreeView->append_column(_("name"), columns.name);
	// Connect the select signal
	refTreeSelection=ElementTreeView->get_selection();
	refTreeSelection->signal_changed().connect(
    sigc::mem_fun(*this, &GuiAddElement::on_selection_changed));
}
GuiAddElement::~GuiAddElement()
{
}
void GuiAddElement::set_path(RefPtr<OpticPath> newpath)
{
	path=newpath;
	update_treemodel();
}
void GuiAddElement::update_treemodel()
{
	Properties p;
	refElementTreeModel->clear();
	for (guint i=0; i<path->size(); i++)
	{
		row = *(refElementTreeModel->append());
  		row[columns.id] = i;
		p=path->get_Properties(i);
		row[columns.name] = p.get("name").get_string();
		row[columns.type] = p.get("element type").get_string();
	}
}
void GuiAddElement::on_add_clicked()
{
	Gtk::Button* b;
	refXml->get_widget("GuiAddElement::add",b);
	b->set_sensitive(false);
	Gtk::VBox vbox;
	Gtk::Button button_distance(_("New Distance")), 
		button_lens(_("New Lens")),
		button_cancel(_("Cancel"));
	selectwin.add(vbox);
	vbox.pack_start(button_distance);
	vbox.pack_start(button_lens);
	vbox.pack_start(button_cancel);
	button_distance.signal_clicked().connect(
		sigc::mem_fun(*this,&GuiAddElement::button_distance_clicked));
	button_lens.signal_clicked().connect(
		sigc::mem_fun(*this,&GuiAddElement::button_lens_clicked));
	button_cancel.signal_clicked().connect(
		sigc::mem_fun(*this,&GuiAddElement::button_cancel_clicked));								   
	selectwin.show_all_children();
	Gtk::Main::run(selectwin);
}
void GuiAddElement::button_distance_clicked()
{
	RefPtr<OpticElement> element=Distance::create();
	addElement(element);
	selectwin.hide();
	Gtk::Button* b;
	refXml->get_widget("GuiAddElement::add",b);
	b->set_sensitive(true);
}
void GuiAddElement::button_lens_clicked()
{
	RefPtr<OpticElement> element=Lens::create();
	addElement(element);
	selectwin.hide();
	Gtk::Button* b;
	refXml->get_widget("GuiAddElement::add",b);
	b->set_sensitive(true);
}
void GuiAddElement::button_cancel_clicked()
{
	selectwin.hide();
	Gtk::Button* b;
	refXml->get_widget("GuiAddElement::add",b);
	b->set_sensitive(true);
}
void GuiAddElement::addElement(RefPtr<OpticElement> element)
{
	path->add(element);
	update_treemodel();
	Gtk::Button* rmbutton;
	refXml->get_widget("GuiAddElement::remove",rmbutton);
	if (path->size()>0)
	{
		rmbutton->set_sensitive(true);
	}
	else
	{
		rmbutton->set_sensitive(false);
	}
	signalPathChanged_.emit();
}
void GuiAddElement::on_remove_clicked()
{
	Gtk::Button* rmbutton;
	refXml->get_widget("GuiAddElement::remove",rmbutton);
	Gtk::TreeModel::iterator iter = refTreeSelection->get_selected();
	if(iter)
	{
		Gtk::TreeModel::Row row = *iter;
		path->remove(row[columns.id]); // remove element with selected id
		update_treemodel();
		proptree->clear();
	}
	
	if (path->size()<=0)
	{
		rmbutton->set_sensitive(false);
	}
	signalPathChanged_.emit();
}
void GuiAddElement::on_selection_changed()
{
	Gtk::TreeModel::iterator iter = refTreeSelection->get_selected();
	if(iter)
	{
		Gtk::TreeModel::Row row = *iter;
		int elementnr=row[columns.id];
		Properties p=path->get_Properties(elementnr);
		proptree->set(p);
	}
}
void GuiAddElement::on_prop_changed(Properties prop)
{
	PropValue pval;
	Gtk::TreeModel::iterator iter = refTreeSelection->get_selected();
	if(iter)
	{
		Gtk::TreeModel::Row row = *iter;
		int elementnr=row[columns.id];
		for (PropIter i=prop.begin(); i!=prop.end(); i++)
		{
			pval=i->second;
			// only change non-readonly items
			if (!pval.readonly())
			{
				path->set_Property(elementnr,i->first,i->second);
			}
		}
		// update row data (elementname)
		Properties prop=path->get_Properties(elementnr);
		row[columns.name]=prop.get("name").get_string();
		signalPathChanged_.emit();	
	}
}
GuiAddElement::signal_path_changed_t& GuiAddElement::signalPathChanged()
{
	return signalPathChanged_;
}
