/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * LaserCalc
 * Copyright (C) Udo Eisenbarth 2008 <udo.eisenbarth@web.de>
 * 
 * LaserCalc is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * LaserCalc is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "matcher.hpp"
#include "permutation.hpp"
#include "random-gen.hpp"
#include "distance.hpp"
#include <iostream>
#include <stdexcept>
#include <algorithm> // sorting the solution list

using namespace std;

Matcher::Matcher()
{
	status=-1.0; // we are starting in idle mode
}
Matcher::~Matcher()
{
}
void Matcher::set_Input(OpticBeam input)
{
	this->input=input;
}
void Matcher::set_Exit(OpticBeam exit)
{
	this->exit=exit;
}
void Matcher::set_Distance(double distance)
{
	if(distance>0.0)
	{
		this->distance=distance;
	}
	else
		throw std::invalid_argument(_("Matcher::set_Distance length must be >0"));
}
void Matcher::set_MaxElements(guint maxelements)
{
	this->maxelements=maxelements;
}
void Matcher::set_ElementList(RefPtr<OpticPath> newelementlist)
{
	opticlist=*newelementlist;
}
void Matcher::solve()
{
	OpticBeam test;
	vector<OpticPath> paths;
	t_MatcherSolution solution;
	gdouble dist;
	guint j;
	stringstream ss;
	Glib::ustring name;
	status=0.0;
	paths=generate_paths(maxelements);
	// clear (possible) results from earlier runs.
	solutionlist.clear();
	for (guint i=0; i<paths.size(); i++)
	{
		cout << "Config #"<<i<<"/"<<paths.size()<<"  ";
		paths.at(i).print_Path();
		cout << "  ";
		solution=optimize(paths.at(i));
		status=(gdouble)(i+1)/paths.size();
		test=input;
		test.transform(solution.path);
		test.print();
		solution.exit_z=test.get_Z();
		solution.exit_w=test.get_W();
		solution.exit=test;
		solution.input=input;
		// set the name labels for the calculated distances
		// (used for display in path drawing)
		for (j=0; j<solution.path.size(); j=j+2)
		{
			dist=solution.path.get_Properties(j).get("length").get_double();
			ss.str("");
			ss.clear();
			ss << dist;
			name=ss.str();
			solution.path.set_Property(j,"name",PropValue(name));
		}
		solutionlist.push_back(solution);
	}
	stable_sort(solutionlist.begin(), solutionlist.end(), compare_solutions);
	status=-1.0;
}
bool Matcher::data_valid()
{
	if (opticlist.size()>0)
		return true;
	else return false;
}
double Matcher::get_status()
{
	return status;
}
t_MatcherSolution Matcher::optimize(OpticPath path)
{
	gdouble dist;
	guint i;
	gdouble decrease=1.0001;
	gdouble temperature=100.0;
	gdouble maxstep=50.0;
	gdouble stepdecrease=1.00001;
	gdouble penalty, testpenalty,prob;
	t_MatcherSolution solution;
	OpticPath testpath;
	vector<double> testparams;
	vector<double> params; // variation parameters
	// extract variation parameters(=distances) of the path
	for (i=0; i<path.size(); i=i+2)
	{
		dist=path.get_Properties(i).get("length").get_double();
		params.push_back(dist);
	}
	penalty=evaluate_path(path);
	// Here starts the annealing
	while (temperature>0.000001)
	{
		//cout << "Temp: " << temperature;
		/*
		for (i=0; i<params.size(); i++)
			cout << " " << params[i];
		cout << " Penalty " << penalty << endl;
		*/
		// Try variation
		testparams=vary_params(params,maxstep);
		testpath=update_path(path,testparams);
		testpenalty=evaluate_path(testpath);
		prob=exp(-(testpenalty-penalty)/temperature);
		if (prob>rnd.get())
		{
			path=testpath;
			params=testparams;
			penalty=testpenalty;
		}
		temperature=temperature/decrease;
		maxstep=maxstep/stepdecrease;
	}
	solution.penalty=penalty;
	solution.path=testpath;
	cout << "penalty: " << penalty;
	cout << "    Params:";
	for (i=0; i<params.size(); i++)
		cout << " "<<params.at(i);
	cout<< "     ";
	
	return solution;
}
vector<double> Matcher::vary_params(vector<double> params,double maxvariation)
{
	double sum=0;
	double newparam;
	vector<double> testparams;
	
	do
	{
		testparams=params;
		sum=0;
		for (guint i=0; i<testparams.size()-1; i++)
		{
			do
			{
				newparam=testparams.at(i)+rnd.get_sym(maxvariation);			
			} while (newparam<0.0);
			sum=sum+newparam;
			testparams.at(i)=newparam;
		}
	} while (sum>distance);
	params=testparams;
	params.at(params.size()-1)=distance-sum;
	return params;
}
OpticPath Matcher::update_path(OpticPath path, vector<double> params)
{
	for(guint i=0; i<params.size(); i++)
	{
		path.set_Property(2*i,"length",PropValue(params.at(i)));
	}
	return path;
}
t_MatcherSolutionList Matcher::get_solution()
{
	return solutionlist;
}
double Matcher::evaluate_path(OpticPath path)
{
	OpticBeam temp=input;
	double z1,z2,w1,w2;
	double result;
	double weight=1000.0;
	temp.transform(path);
	z1=exit.get_Z(); z2=temp.get_Z();
	w1=exit.get_W(); w2=temp.get_W();
	result=abs((z1-z2))+weight*abs((w1-w2));
	return(result);
}
vector<OpticPath> Matcher::generate_paths(guint maxelements)
{
	vector<OpticPath> paths;
	Permutation permutation=Permutation(1,1);
	t_permlist plist;
	t_perm perm;
	for (guint i=1;i<=maxelements; i++)
	{
		permutation=Permutation(i,opticlist.size());
		plist=permutation.get();
		for (guint plist_ind=0; plist_ind<plist.size(); plist_ind++)
		{
			perm=plist.at(plist_ind);
			// build individual path
			OpticPath path=OpticPath();
			for (guint perm_ind=0; perm_ind<perm.size(); perm_ind++)
			{
				path.add(Distance::create(distance/(i+1)));
				path.add(opticlist.get_Element(perm[perm_ind]));
			}
			path.add(Distance::create(distance/(i+1)));
			paths.push_back(path);
		}
	}
	return paths; //Does this work? All members of path should be auto-deleted???
}
void Matcher::print_Elements()
{
	for (guint i=0; i<opticlist.size(); i++)
		(opticlist.get_Properties(i)).print();
}
bool compare_solutions(t_MatcherSolution s1, t_MatcherSolution s2)
{
        return(s1.penalty<s2.penalty);
}
