/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * LaserCalc
 * Copyright (C) Udo Eisenbarth 2008 <udo.eisenbarth@web.de>
 * 
 * LaserCalc is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * LaserCalc is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "path-plot.hpp"
#include <iostream>

PathPlot::PathPlot(BaseObjectType* cobject,
			 const Glib::RefPtr<Gnome::Glade::Xml>& glade_xml)
:Gtk::DrawingArea(cobject)
{
	glade_xml->get_widget("PathPlot::xlabel",xlabel);
	glade_xml->get_widget("PathPlot::ylabel",ylabel);
	// set event mask in order to evaluate mouse movements
	add_events( Gdk::POINTER_MOTION_MASK );
	//| Gdk::POINTER_MOTION_HINT_MASK |
  	//Gdk::BUTTON_PRESS_MASK | Gdk::BUTTON_RELEASE_MASK );
}
PathPlot::~PathPlot()
{
}
void PathPlot::display(OpticPath newpath)
{
	path=newpath;
	path_to_data(200);
	on_expose_event(0);
}
bool PathPlot::on_expose_event(GdkEventExpose* event)
{
	if (data.size()>0) // do something if data available
	{
		guint i;
		// This is where we draw on the window
		Glib::RefPtr<Gdk::Window> window = get_window();
		if(window)
		{
			window->clear(); // clear the drawing area (Gdk::Window)
			Gtk::Allocation allocation = get_allocation();
			const int width = allocation.get_width();
			const int height = allocation.get_height();
			Cairo::RefPtr<Cairo::Context> cr = window->create_cairo_context();
			//
			Glib::RefPtr<Pango::Layout> pangoLayout = Pango::Layout::create (cr);
			Pango::FontDescription font_descr( "sans Ultra-Light 10" );
    		pangoLayout->set_font_description( font_descr);			
			cr->set_line_width(1.0);
			// clip to the area indicated by the expose event so that we only redraw
			// the portion of the window that needs to be redrawn
			if (event)
			{
				cr->rectangle(event->area.x, event->area.y,
							event->area.width, event->area.height);
				cr->clip();
			}
			plot_elements(width,height,cr, pangoLayout);
			cr->set_source_rgb(0.8, 0.0, 0.0);
			const double scale_x=width/data.get_MaxX();
			const double scale_y=0.9*height/data.get_MaxY(); //scale to 90% of window
			cr->move_to((data.get(0).x*scale_x),(0.95*height)-(data.get(0).y*scale_y));
			for (i=0; i<data.size(); i++)
			{
				cr->line_to((data.get(i).x*scale_x),(0.95*height)-(data.get(i).y*scale_y));
			}
        	cr->stroke();
		}
  }
  return true;
}
bool PathPlot::on_motion_notify_event(GdkEventMotion* event)
{
	double position;
	string labelstring;
	stringstream ss;
	Glib::RefPtr<Gdk::Window> window = get_window();
	if(window)
	{
		Gtk::Allocation allocation = get_allocation();
		const int width = allocation.get_width();
		//const int height = allocation.get_height();
		position=event->x*(path.get_PathLength()/width);
		ss << position;
		ss >> labelstring;
		ss.clear();
		xlabel->set_text(labelstring);
		ss << get_path_radius(position);
		ss >> labelstring;
		ss.clear();
		ylabel->set_text(labelstring);
	}
	return true;
}
double PathPlot::get_path_radius(double position)
{
	OpticPath temppath;
	OpticBeam tempbeam;
	int elementnr=0;
	double templength=0.0;
	if (position<path.get_PathLength())
	{
		// calculate temporary path up to position
		while (position>=templength)
		{
			temppath.add(path.get_Element(elementnr));
			templength=temppath.get_PathLength();
			elementnr++;
		}
	}
	else
	{
		// longer than pathlength-> take full path
		temppath=path;
		templength=path.get_PathLength();
	}
	tempbeam=input;
	tempbeam.transform(temppath);
	return(tempbeam.get_Radius(position-templength));
}
void PathPlot::path_to_data(guint nr_of_points)
{
	gdouble position_step, position;
	guint pointnr;
	t_pointset tempdata;
	t_datapoint tempdatapoint;
	
	position_step=path.get_PathLength()/(nr_of_points-1);
	
	position=0.0;
	for (pointnr=0; pointnr<nr_of_points; pointnr++)
	{
		position=pointnr*position_step;
		tempdatapoint.x=position;
		tempdatapoint.y=get_path_radius (position);
		tempdata.push_back(tempdatapoint);
	}
	data.add(tempdata);
}
void PathPlot::set_Input(OpticBeam newinput)
{
	input=newinput;
}
/*
void PathPlot::set_Input(RefPtr<OpticBeam> newinput)
{
	input=*newinput;
}
*/
void PathPlot::plot_elements(guint width,guint height,
	Cairo::RefPtr<Cairo::Context> cr, Glib::RefPtr<Pango::Layout> pangoLayout)
{
	gdouble scale_x=width/path.get_PathLength();
	gdouble position=0.0;
	gdouble length;
	gint twidth,theight;
	Glib::ustring name;
	
	cr->save();
	cr->set_source_rgb(0.0, 0.0, 0.0);
	for(guint i=0; i<path.size(); i++)
	{
		length=path.get_Properties(i).get("path length").get_double();
		name=path.get_Properties(i).get("name").get_string();
		pangoLayout->set_text(name);
		// get size of text
		pangoLayout->get_size(twidth,theight);
		// convert from PangoUnits to screen units
		twidth=twidth/Pango::SCALE;
		theight=theight/Pango::SCALE;
		// print the element label
		if (length<0.1)
		{
			cr->move_to((position*scale_x)+3,height-theight-2);
		}
		else
		{
			cr->move_to((((position+length/2.0)*scale_x)-twidth/2.0),height-theight-2);
		}
		pangoLayout->update_from_cairo_context(cr);  //gets cairo cursor position
		pangoLayout->add_to_cairo_context(cr);
		//
		if (i<path.size()-1) // don't draw the last element line
		{
			position=position+length;
			if (length>0.1) // don't draw elements with no/little length
			{
				cr->move_to(position*scale_x,0);
				cr->line_to(position*scale_x,height);
			}
		}
	}
	cr->stroke();
	cr->restore();
}
