/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * treeview
 * Copyright (C) Udo Eisenbarth 2008 <udo.eisenbarth@web.de>
 * 
 * treeview is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * treeview is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "prop-renderer.hpp"
#include <iostream>
#include <sstream>

PropRenderer::PropRenderer():
	Glib::ObjectBase(typeid(PropRenderer)),
	Gtk::CellRenderer(),
	property_value_(*this,"propvalue")
{
	property_mode() = Gtk::CELL_RENDERER_MODE_EDITABLE;
}
PropRenderer::~PropRenderer()
{
}
Glib::PropertyProxy<PropValue> PropRenderer::property_value()
{
	return property_value_.get_proxy();
}
void PropRenderer::registerTypeRenderer(const Valtype& type_id, cell_renderer_ptr_t renderer_ptr, bool is_default/*=false*/)
{
    if (type_renderer_map_.find(type_id) != type_renderer_map_.end())
        throw std::runtime_error ( "A CellRenderer was already registered for type: " + type_id );
    
    // TODO: who manage the pointer?
    type_renderer_map_.insert (std::make_pair (type_id, renderer_ptr));
    
    if (is_default)
        default_renderer_ = type_id;
}

void PropRenderer::selectDefaultTypeRenderer(const Valtype& type_id)
{
    default_renderer_ = type_id;
}

PropRenderer::cell_renderer_ptr_t PropRenderer::getTypeRenderer(const Valtype& type_id) const
{
    // Search for the CellRenderer to use
    if (type_renderer_map_.find (type_id) != type_renderer_map_.end())
    {
        // Use registered CellRenderer
        return type_renderer_map_.at(type_id);
    }
    else
    {
        // Use default CellRenderer
        if (type_renderer_map_.find (default_renderer_) != type_renderer_map_.end())
        {
            return type_renderer_map_.at(default_renderer_);
        }
    }
    
    // TODO: Critical, throw an exception? 
    return 0;
}

void PropRenderer::unregisterTypeRenderer(const Valtype& type_id)
{
    if (type_renderer_map_.find(type_id) == type_renderer_map_.end())
        throw std::runtime_error ( "Non existent CellRenderer registered for type: " + type_id );
    
    type_renderer_map_.erase (type_id);
    
    if (default_renderer_ == type_id)
        default_renderer_=NONE;
}

void PropRenderer::unregisterAll()
{
    type_renderer_map_.clear();
    default_renderer_=NONE;
}
void PropRenderer::notifyEdited(const Glib::ustring& path_string, PropValue& value)
{
	signal_edited_.emit (path_string, value);
}
void PropRenderer::get_size_vfunc (Gtk::Widget& widget, const Gdk::Rectangle* cell_area, int* x_offset, int* y_offset, int* width, int* height) const
{
	PropValue prop=property_value_.get_value();
	Valtype type_id=prop.get_type();
    getTypeRenderer(type_id)->get_size_vfunc (widget, cell_area, x_offset, y_offset, width, height);
}

void PropRenderer::render_vfunc (const Glib::RefPtr<Gdk::Drawable>& window, Gtk::Widget& widget, const Gdk::Rectangle& background_area, const Gdk::Rectangle& cell_area, const Gdk::Rectangle& expose_area, Gtk::CellRendererState flags)
{
	PropValue prop=property_value_.get_value();
	Valtype type_id=prop.get_type();
	getTypeRenderer(type_id)->set_value(prop);
	getTypeRenderer(type_id)->render_vfunc (window, widget, background_area, cell_area, expose_area, flags);
}

bool PropRenderer::activate_vfunc (GdkEvent* event, Gtk::Widget& widget, const Glib::ustring& path, const Gdk::Rectangle& background_area, const Gdk::Rectangle& cell_area, Gtk::CellRendererState flags)
{
	PropValue prop=property_value_.get_value();
	Valtype type_id=prop.get_type();
    return getTypeRenderer(type_id)->activate_vfunc (event, widget, path, background_area, cell_area, flags);
}

Gtk::CellEditable* PropRenderer::start_editing_vfunc(GdkEvent* event, Gtk::Widget& widget, const Glib::ustring& path, const Gdk::Rectangle& background_area, const Gdk::Rectangle& cell_area, Gtk::CellRendererState flags)
{
	PropValue prop=property_value_.get_value();
	Valtype type_id=prop.get_type();
	getTypeRenderer(type_id)->set_value(prop);
    return getTypeRenderer(type_id)->start_editing_vfunc(event, widget, path, background_area, cell_area, flags);
}

PropRenderer::signal_edited_t& PropRenderer::signalEdited()
{
	return signal_edited_;
}
